using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Text;
using System.Windows.Forms;
using GMAS_Lib_test;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace MMCLibDotNET_Test_2010
{
    public partial class DownloadControl : UserControl, IParentPanel
    {
        public DownloadControl()
        {
            InitializeComponent();
        }

        private Panel mainPanel;
        public Panel GetPanel()
        {
            return mainPanel;
        }

        private ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            mainPanel = pnl;
        }

        private void buttonBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Firmware (*.gms)|*.gms|All Files (*.*)|*.*";
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            textBoxPath.Text = dlg.FileName;
        }

        private void buttonDown_Click(object sender, EventArgs e)
        {
            if (System.IO.File.Exists(textBoxPath.Text) == false)
            {
                MessageBox.Show("Invalid path " + textBoxPath.Text, "FW Download");
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "FW Download");
                return;
            }
            if (MMCConnection.DownloadFirmwareFile(ConnectData.Handle, 
                textBoxPath.Text, OnStart, OnProgress, OnFinish) == false)
            {
                MessageBox.Show("Download Firmware can not be started", "FW Download");
                return;
            }
        }

        private void OnStart(int handle, LoadType type)
        {
            ConnectData.ToOutput("FW Download started!");
            iterator = 0;
        }

        private void OnProgress(int comHandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("FW Download: stage - {0} {1}", stage, dataSize));
            Application.DoEvents();
        }

        int iterator;
        private void OnFinish(int comHandle, LoadType type, Error error)
        {
            //if (iterator >= model.TargetsData.Count)
            //    return;

            if (error != Error.OK)
            {
                ConnectData.ToOutput(string.Format("FW Download Failed: {0}", error));
                return;
            }
            ConnectData.ToOutput(string.Format("FW Download OK: {0}", iterator));

            //WaitGmasToConnect(trg);
            iterator++;
        }

        private UserFileTransferDictionary files;

        private Stopper stopper;

        private void buttonDownloadUF_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "UF Transfer");
                return;
            }
            stopper = new Stopper(true);
            if (File.Exists(textBoxDirUF.Text))
            {
                files = null;
                if (MMCConnection.DownloadUserFile(ConnectData.Handle,
                    textBoxDirUF.Text, OnUFStart, OnUFProgress, OnUFFinish) == false)
                {
                    MessageBox.Show("Download User File can not be started", "UF Transfer");
                }
            }
            else
            {
                if (Directory.Exists(textBoxDirUF.Text) == false)
                {
                    MessageBox.Show("Invalid root " + textBoxDirUF.Text);
                    return;
                }
                files = new UserFileTransferDictionary();
                foreach (object item in listFiles.Items)
                {
                    files.AddFileAsName(item as string);
                }
                if (MMCConnection.DownloadUserFiles(ConnectData.Handle,
                    textBoxDirUF.Text, files, OnUFStart, OnUFProgress, OnUFFinish) == false)
                {
                    MessageBox.Show("Download User Files can not be started", "UF Transfer");
                    return;
                }
            }
        }

        private void OnUFFinish(int comhandle, LoadType type, Error error)
        {
            ConnectData.ToOutput(string.Format("UF Transfer : {0} ; Elapsed : {1} ms",
                                               error, stopper.StopEx()));
            if (files != null)
            {
                foreach (KeyValuePair<string, UserFileTransferStatus> pair in files)
                {
                    ConnectData.ToOutput(string.Format("File {0} - {1}", pair.Key, pair.Value));
                }
            }
        }

        private void OnUFProgress(int comhandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("UF Transfer: stage - {0} {1}", stage, dataSize));
        }

        private void OnUFStart(int handle, LoadType type)
        {
            ConnectData.ToOutput("UF Transfer started!");
        }

        private void buttonUploadUF_Click(object sender, EventArgs e)
        {
            if (Directory.Exists(textBoxDirUF.Text) == false)
            {
                MessageBox.Show("Invalid root " + textBoxDirUF.Text);
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "UF Transfer");
                return;
            }
            files = new UserFileTransferDictionary();
            foreach (object item in listFiles.Items)
            {
                files.AddFileAsName(item as string);
            }
            if (MMCConnection.UploadUserFiles(ConnectData.Handle,
                textBoxDirUF.Text, files, OnUFStart, OnUFProgress, OnUFFinish) == false)
            {
                MessageBox.Show("Upload User Files can not be started", "UF Transfer");
                return;
            }
        }
    }
}
