using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using GMAS_Lib_test;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace MMCLibDotNET_Test_2010
{
    public partial class DriveFWPAL : UserControl, IParentPanel
    {
        public DriveFWPAL()
        {
            InitializeComponent();
        }
        private Panel mainPanel;
        public Panel GetPanel()
        {
            return mainPanel;
        }

        private ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            mainPanel = pnl;
        }
        #region FOEEvent
        public bool FoeEvent(ref FOEStatus data)
        {
            if (data == null)
            {//communication failure.
                MessageBox.Show("The EAS lost connection with the G-MAS.", "FOE Download");
                return false;
            }
            for (int i = 0; i < data.ucNumOfSlaves; i++)
            {
                FOE_SlaveInfo info = data.pstSlavesErrorID[i];
                if (info.sErrorID != 0)
                {
                    MessageBox.Show("Check Error according GMAS personality", "FOE Download");
                }
                else
                {
                    ConnectData.ToOutput(string.Format("FOEStatus: {0}", 100 - data.ucProgress));
                }
            }

            return true;
        }
 
        #endregion FOEEvent
        #region DLFirmwareByFOE
        private void buttonBrowseFW_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Firmware (*.abs)|*.abs";
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            textBoxPath.Text = dlg.FileName;
        }

        private void buttonDownFW_Click(object sender, EventArgs e)
        {
            if (System.IO.File.Exists(textBoxPath.Text) == false)
            {
                MessageBox.Show("Invalid path " + textBoxPath.Text, "Firmware Download");
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "Firmware Download");
                return;
            }
            
            byte slaveNum = 1;// for test purpose only
            ushort[] idsToDownload = new ushort[slaveNum];// array of Drives ID
            ushort driveID = ConnectData.SingleAxis.DriveID; //EACT drive ID
            idsToDownload[0] = driveID;// for test purpose only
            GetFOEStatus foeStatus = new GetFOEStatus(FoeEvent);

            if (MMCConnection.DownloadFOE_DriveFW(ConnectData.Handle, textBoxPath.Text, slaveNum, idsToDownload,
                                              OnStartFW, OnProgressFW, OnFinishFW,foeStatus) == false)                                      
            {
                MessageBox.Show("Download Firmware by FOE can not be started", "FW Download");
                return;
            }
        }
        private void OnStartFW(int handle, LoadType type)
        {
            ConnectData.ToOutput("Download Firmware by FOE started!");
 
        }
        private void OnProgressFW(int comHandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("Download Firmware by FOE: stage - {0} {1}", stage, dataSize));
            Application.DoEvents();
        }
        private void OnFinishFW(int comHandle, LoadType type, Error error)
        {

            if (error != Error.OK)
            {
                ConnectData.ToOutput(string.Format("Download Firmware by FOE Failed: {0}", error));
                return;
            }
            ConnectData.ToOutput("Download Firmware by FOE OK: {0}");
            //Then this event is received the application has to wait then drive will be returnes to normal functionality
            //The best way check operation mode in separate thread during few seconds
        }

#endregion DLFirmwareByFOE

#region DLPALByFOE
        private void buttonBrowsePAL_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "PAL (*.pbin)|*.pbin";
            if (dlg.ShowDialog() != DialogResult.OK)
                return;
            textBoxPathPAL.Text = dlg.FileName;
        }
        private void buttonDownloadPAL_Click(object sender, EventArgs e)
        {
            if (System.IO.File.Exists(textBoxPathPAL.Text) == false)
            {
                MessageBox.Show("Invalid path " + textBoxPathPAL.Text, "PAL Download");
                return;
            }
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "PAL Download");
                return;
            }

            byte slaveNum = 1;// for test purpose only
            ushort[] idsToDownload = new ushort[slaveNum];// array of Drives ID
            ushort driveID = ConnectData.SingleAxis.DriveID; //EACT drive ID
            idsToDownload[0] = driveID;// for test purpose only
            GetFOEStatus foeStatus = new GetFOEStatus(FoeEvent);

            if (MMCConnection.DownloadFOE_PAL(ConnectData.Handle, textBoxPathPAL.Text, slaveNum, idsToDownload,
                                              OnStartPAL, OnProgressPAL, OnFinishPAL, foeStatus) == false)
            {
                MessageBox.Show("Download Firmware by FOE can not be started", "PALDownload");
                return;
            }

        }
        private void OnStartPAL(int handle, LoadType type)
        {
            ConnectData.ToOutput("Download PAL by FOE started!");

        }
        private void OnProgressPAL(int comHandle, LoadType type, uint dataSize, Stage stage)
        {
            ConnectData.ToOutput(string.Format("Download PAL by FOE: stage - {0} {1}", stage, dataSize));
            Application.DoEvents();
        }
        private void OnFinishPAL(int comHandle, LoadType type, Error error)
        {

            if (error != Error.OK)
            {
                ConnectData.ToOutput(string.Format("Download PAL by FOE Failed: {0}", error));
                return;
            }
            ConnectData.ToOutput("Download PAL by FOE OK: {0}");
            //Then this event is received the application has to wait then drive will be returnes to normal functionality
            //The best way check operation mode in separate thread during few seconds
        }

#endregion DLPALByFOE
    }
}
