﻿using System;
using System.Windows.Forms;
using System.Net;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            Array ar = Enum.GetValues( typeof(MMC_BOOLEAN_PARAMETER_LIST_ENUM) );
            foreach( object obj in ar)
            comboBox1.Items.Add( obj );
            comboBox1.SelectedIndex = 0;

            Array arOPM = Enum.GetValues( typeof(OPM402) );
            foreach (object obj in arOPM)
                cmbOpMode.Items.Add(obj);
            cmbOpMode.SelectedIndex = 0;
        }

        //public delegate void cbFunc(object sender, MMC_CAN_REPLY_DATA_OUT data);
        private void UserCallback( object sender, MMC_CAN_REPLY_DATA_OUT data )
        {
            
        }

        /// <summary>
        /// Buttons the connect click. Call function "MMCConnection.ConnectRPC(...) "
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void ButtonConnectClick(object sender, EventArgs e)
        {
           
           IPAddress rIP;
            if (!IPAddress.TryParse(tb_RIP.Text, out rIP))
            {
                MessageBox.Show("IP Format Error:Wrong Remote IP", "IP Format Error");
                return;
            }
            
            IPAddress lIP;
            if (!IPAddress.TryParse(tb_LIP.Text, out lIP))
            {
                MessageBox.Show("IP Format Error:Wrong Local IP", "IP Format Error");
                return;
            }
            if (MMCConnection.ConnectRPC(rIP, 4000, lIP, 5000, new cbFunc(UserCallback), 0xEFFFFFFF, out hndl) != 0)
            {
                MessageBox.Show("Connection failed. Check network configuration.", "Connection Failed");
                return;
            }
            String str = "Connection successfully established.";
            listBox1.Items.Clear();
            listBox2.Items.Clear();
            listBox1.Items.Add(str);
            listBox2.Items.Add(str);

        }

        /// <summary>
        /// Buttons the get version click. Call function "MMCConnection.GetVersion(...) "
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonGetVersionClick(object sender, EventArgs e)//Version
        {
            UInt32 boot;
            String ver = string.Empty;
            try
            {
                MMCConnection.GetVersion(hndl, out boot, out ver);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            listBox1.Items.Add( "Boot version = " + boot.ToString() + " FW version = " + ver );
        }

        /// <summary>
        /// Buttons the disconnect click.Call function " MMCConnection.CloseConnection(...) "
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonDisconnectClick(object sender, EventArgs e)//Disconnect
        {
            
            try
            {
                MMCConnection.CloseConnection(MMCConnection.GetConnection(hndl));
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String str = "Disconnected.";
            
            listBox1.Items.Add(str);
        }

        /// <summary>
        /// Buttons the get Axis reference click.
        ///     1.To do it create new object of MMCSingleAxis 
        ///     2. get properties  sa.AxisReference
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonGetRefClick(object sender, EventArgs e)
        {
            try
            {
                sa = new MMCSingleAxis(tbAxName.Text, hndl);//create new object of MMCSingleAxis 
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            listBox1.Items.Add("Axis Reference of Axis " + tbAxName.Text + " = " + sa.AxisReference.ToString() + ". DriveID of the Axis = " + sa.DriveID.ToString());//get properties  sa.AxisReference
            
        }

        /// <summary>
        /// Buttons the AcceptActPos click. (the example measure time of update actual position)
        /// call function GetActualPosition();
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ButtonPosAcceptClick(object sender, EventArgs e)
        {
            double tm = 0;
            double pos = -1;
            Stopper st = new Stopper();
            try
            {
               for (int i = 0; i < 100; i++)
                {
                    st.DebugStart();
                    pos = sa.GetActualPosition();//call function GetActualPosition();
                    st.DebugStop();
                    tm += st.Interval;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
 
            String res = "Average time for GetActualPosition = " + (tm/100).ToString() + " for local EP " + tb_LIP.Text;
            listBox1.Items.Add(res);


        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
                
        }

        private void ButtonReadStatustClick(object sender, EventArgs e)
        {
            UInt32 status;
            try
            {
                status = sa.ReadStatus();
            }
            catch( MMCException ex )
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Status of Axis " + sa.AxisName + " successfully accepted. Status Value = 0x" + status.ToString("X");
            listBox1.Items.Add(res);
        }

        private void ButtonReadBoolParamClick(object sender, EventArgs e)
        {
            MMC_BOOLEAN_PARAMETER_LIST_ENUM param = (MMC_BOOLEAN_PARAMETER_LIST_ENUM)comboBox1.SelectedItem;
            UInt32 Value;
            try
            {
                Value = sa.GetBoolParameter(param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Value of Parameter " + param.ToString() + " successfully accepted. Value = " + Value.ToString();
            listBox1.Items.Add(res);
        }

        private void ButtonReadAxisErrorClick(object sender, EventArgs e)
        {
            ushort AxErr = 0;
            ushort emergCode = 0xFFFF;
            try
            {
                AxErr = sa.GetAxisError(ref emergCode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Axis Error successfully accepted. ErrorValue = " + AxErr.ToString() +". EmergCode = " + emergCode.ToString();
            listBox1.Items.Add(res);
        }

        private void ButtonChngOpModeClick(object sender, EventArgs e)
        {
            OPM402 mode = (OPM402)cmbOpMode.SelectedItem;
            try
            {
                sa.SetOpMode(mode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Operated Mode successfully changed to " + mode.ToString();
            listBox1.Items.Add(res);
        }

        private void ButtonResetClick(object sender, EventArgs e)
        {
            try
            {
                sa.Reset();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Reset command successfully completed. Wait 30 sec and try Connect again";
            listBox1.Items.Add(res);
        }

        private void ButtonPowerOnClick(object sender, EventArgs e)
        {
            try
            {
                sa.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOn command successfully completed.";
            listBox1.Items.Add(res);
        }

        private void ButtonPowerOffClick(object sender, EventArgs e)
        {
            try
            {
                sa.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOff command successfully completed.";
            listBox1.Items.Add(res);
        }

        private void ButtonMoveAbsoluteClick(object sender, EventArgs e)
        {
            int val = -1;
            double pos = 2000;
            try
            {
                val = sa.MoveAbsolute(pos, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveAbsolute command successfully Accepted.";
            listBox1.Items.Add(res);
        }

        private void ButtonMoveVelocityClick(object sender, EventArgs e)
        {
            int val = -1;
            float velo = 2000;
            try
            {
                val = sa.MoveVelocity(velo, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "MoveVelocity command successfully Accepted.";
            listBox1.Items.Add(res);
        }

        private void ButtonStopClick(object sender, EventArgs e)
        {
            try
            {
                sa.Stop(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Stop command successfully Accepted.";
            listBox1.Items.Add(res);
        }

        private void ButtonHomeClick(object sender, EventArgs e)
        {
            short err = 0;
            UInt32 FB_hndl= 0;
            try
            {
                sa.Home(ref err, ref FB_hndl);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Home command successfully Accepted. FB handle = " + FB_hndl.ToString();
            listBox1.Items.Add(res);
        }

        private void ButtonExecuteLabelClick(object sender, EventArgs e)
        {
            byte[] lblArr = new byte[ tbLabel.Text.Length ];
            lblArr.Initialize();
            for (int i = 0; i <lblArr.Length; i++)
            {
                lblArr[i] = (byte)tbLabel.Text[i];
            }
            sa.ElmoExecute(lblArr, (byte)lblArr.Length);
        }

        private void ButtonSetArrClick(object sender, EventArgs e)
        {
            byte[] lblArrName = new byte[tbArrName.Text.Length];
            lblArrName.Initialize();
            for (int i = 0; i < lblArrName.Length; i++)
            {
                lblArrName[i] = (byte)tbArrName.Text[i];
            }
            short index = Int16.Parse(tbArrIndex.Text);
            int val = 10;
            sa.ElmoSetAsyncArray(lblArrName, index, ref val);
        }

        private void ButtonPDOConfClick(object sender, EventArgs e)
        {
            try
            {
                sa.ConfigPDO(PDO_NUMBER_ENUM.PDO_NUM_3, PDO_PARAM_TYPE_ENUM.PDO_PARAM_REG, 0, 0, 0, 0, 0, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "ConfigPDO command successfully Accepted. ";
            listBox1.Items.Add(res);
        }

        private void ButtonGetGroupRefClick(object sender, EventArgs e)
        {
            try
            {
                ga = new MMCGroupAxis(tbGroupName.Text, hndl);//create new object of MMCSingleAxis 
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            listBox2.Items.Add("Group Reference of Group " + tbGroupName.Text + " = " + ga.AxisReference.ToString() );
        }

        private void ButtonReadGroupStatusClick(object sender, EventArgs e)
        {
            UInt32 status;
            try
            {
                status = ga.GroupReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Status of Group " + ga.AxisName + " successfully accepted. Status Value = 0x" + status.ToString("X");
            listBox2.Items.Add(res);
        }

        private void ButtonReadGroupErrorClick(object sender, EventArgs e)
        {
            ushort error;
            try
            {
                error = ga.GroupReadError();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Error of Group " + ga.AxisName + " successfully accepted. Error Value = 0x" + error.ToString("X");
            listBox2.Items.Add(res);
        }

        private void ButtonGroupEnableClick(object sender, EventArgs e)
        {
            try
            {
                ga.GroupEnable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Enable Command for Group " + ga.AxisName + " successfully accepted." ;
            listBox2.Items.Add(res);
        }

        private void ButtonGroupDisableClick(object sender, EventArgs e)
        {
            try
            {
                ga.GroupDisable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Disable Command for Group " + ga.AxisName + " successfully accepted.";
            listBox2.Items.Add(res);
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void cmbOpMode_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void label6_Click(object sender, EventArgs e)
        {

        }

        private void tbLabel_TextChanged(object sender, EventArgs e)
        {

        }

        private void tbArrName_TextChanged(object sender, EventArgs e)
        {

        }

        private void tbArrIndex_TextChanged(object sender, EventArgs e)
        {

        }

        private void label4_Click(object sender, EventArgs e)
        {

        }

        private void label5_Click(object sender, EventArgs e)
        {

        }
    }
}
