﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class GroupMonitorControl : UserControl, IParentPanel
    {
        public GroupMonitorControl()
        {
            InitializeComponent();
            Array ar = Enum.GetValues(typeof(MMC_PARAMETER_LIST_ENUM));
            foreach (object obj in ar)
                cmbParameter.Items.Add(obj);
            cmbParameter.SelectedIndex = 0;
        }

        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);

        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);

        }
        private void EnableButtons(bool enable)
        {
            btnGetMembersInfo.Enabled = enable; 
            btnParamRead.Enabled = enable;
            btnParamWrite.Enabled = enable;
            cmbParameter.Enabled = enable;

            btnReadStatus.Enabled = enable;
            btnReadPosition.Enabled = enable;
            btnReadVelocity.Enabled = enable;
            btnReadError.Enabled = enable;
            btnEnableEndMotionEvent.Enabled = enable;
            btnDisableEndMotionEvent.Enabled = enable;
        }
        private void btnReadStatus_Click(object sender, EventArgs e)
        {
            ushort errID = 0;
            UInt32 res = 0;
            try
            {
                res = ConnectData.GroupAxis.GroupReadStatus(ref errID);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            if( errID != 0 )
            {
                ConnectData.ToOutput( "Group Read Status failed. Error = " + errID.ToString());
                return;
            }
            chbStandby.Checked = false; chbDisabled.Checked = false; chbHoming.Checked = false; chbError.Checked = false;
            chbMoving.Checked = false; chbStop.Checked = false;

            String str_state = string.Empty;
            MC_STATE_GROUP state = (MC_STATE_GROUP)res;
            if ((state &  MC_STATE_GROUP.NC_GROUP_STANDBY) == MC_STATE_GROUP.NC_GROUP_STANDBY)
            { chbStandby.Checked = true;
                str_state += (MC_STATE_GROUP.NC_GROUP_STANDBY.ToString() + " & "); }
            if ((state & MC_STATE_GROUP.NC_GROUP_DISABLED) == MC_STATE_GROUP.NC_GROUP_DISABLED){
                chbDisabled.Checked = true;
            str_state += (MC_STATE_GROUP.NC_GROUP_DISABLED.ToString() + " & "); }
            if ((state & MC_STATE_GROUP.NC_GROUP_HOMING) == MC_STATE_GROUP.NC_GROUP_HOMING){
                chbHoming.Checked = true;
                str_state += (MC_STATE_GROUP.NC_GROUP_HOMING.ToString() + " & ");
            }
            if ((state & MC_STATE_GROUP.NC_GROUP_ERROR) == MC_STATE_GROUP.NC_GROUP_ERROR){
                chbError.Checked = true;
                str_state += (MC_STATE_GROUP.NC_GROUP_ERROR.ToString() + " & ");
            }
            if ((state & MC_STATE_GROUP.NC_GROUP_MOVING) == MC_STATE_GROUP.NC_GROUP_MOVING){
                chbMoving.Checked = true;
                str_state += (MC_STATE_GROUP.NC_GROUP_MOVING.ToString() + " & ");
            }
            if ((state & MC_STATE_GROUP.NC_GROUP_STOPPING) == MC_STATE_GROUP.NC_GROUP_STOPPING){
                chbStop.Checked = true;
                str_state += (MC_STATE_GROUP.NC_GROUP_STOPPING.ToString());
            }
            ConnectData.ToOutput("Group Read Status Proceed. Status:" + str_state);    

        }

        private void btnReadPosition_Click(object sender, EventArgs e)
        {
            double[] ps = new double[ 16 ];
            try
            {
                ConnectData.GroupAxis.GroupReadActualPosition(MC_COORD_SYSTEM_ENUM.MC_ACS_COORD, ref ps);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            String str = "Actual Position for Group " + ConnectData.GroupAxis.AxisName + " :";
            for (int i = 0; i < 16; i++  )
            {
                str += " [" + ps[i].ToString() + "]";
            }
            ConnectData.ToOutput(str); 
        }

        private void btnReadError_Click(object sender, EventArgs e)
        {
            ushort groupError;
            try
            {
                groupError = ConnectData.GroupAxis.GroupReadError();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput( "Group Error accepted. Value is " + groupError.ToString());
        }

        private void btnReadVelocity_Click(object sender, EventArgs e)
        {
            double vPath = 0;
            try
            {
                vPath = ConnectData.GroupAxis.GroupReadActualVelocity(MC_COORD_SYSTEM_ENUM.MC_ACS_COORD);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput("Velocity for group " + ConnectData.GroupAxis.AxisName +  " accepted. Velocity Path = " + vPath.ToString());
        }

        private void btnEnable_Click(object sender, EventArgs e)
        {
            
            try
            {
                ConnectData.GroupAxis.GroupEnable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput("GroupEnable command accepted  for group " + ConnectData.GroupAxis.AxisName );
        }

        private void btnDisable_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.GroupAxis.GroupDisable();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }

            ConnectData.ToOutput("GroupDisable command accepted  for group " + ConnectData.GroupAxis.AxisName);
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void btnEnableEndMotionEvent_Click(object sender, EventArgs e)
        {
            
            try
            {
                ConnectData.GroupAxis.EnableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            if (ConnectData.SingleAxis == null)
                return;
            try
            {
                ConnectData.SingleAxis.EnableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnDisableEndMotionEvent_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.GroupAxis.DisableEndMotionEvent();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnParamWrite_Click(object sender, EventArgs e)
        {
            MMC_PARAMETER_LIST_ENUM param = (MMC_PARAMETER_LIST_ENUM)cmbParameter.SelectedItem;
            Double val = Double.Parse(tbParamValue.Text);
            try
            {
                ConnectData.GroupAxis.SetParameter(val, param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnParamRead_Click(object sender, EventArgs e)
        {
            Double res = 0;
            MMC_PARAMETER_LIST_ENUM param = (MMC_PARAMETER_LIST_ENUM)cmbParameter.SelectedItem;
            try
            {
               res =  ConnectData.GroupAxis.GetParameter( param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnGetMembersInfo_Click(object sender, EventArgs e)
        {
            GroupMembersInfo membersInfo = null;
            try
            {
                membersInfo = ConnectData.GroupAxis.GetGroupMembersInfo();
            }
            
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            
        }
    }
}
