﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class GroupSplineControl : UserControl, IParentPanel
    {
        public GroupSplineControl()
        {
            InitializeComponent();
        }

        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            Array arDir = Enum.GetValues(typeof(NC_TRANSITION_MODE_ENUM));
            foreach (object obj in arDir)
                cmbTransMode.Items.Add(obj);

            Array boolParams = Enum.GetValues(typeof(MMC_BOOLEAN_PARAMETER_LIST_ENUM));
            foreach (object obj in boolParams)
                cmbBoolParam.Items.Add(obj);
            cmbBoolParam.SelectedIndex = 0;

            Array param = Enum.GetValues(typeof(MMC_PARAMETER_LIST_ENUM));
            foreach (object obj in param)
                cmbParam.Items.Add(obj);
            cmbParam.SelectedIndex = 0;

        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.GroupAxis == null)
                EnableButtons(false);
            else
            {
                EnableButtons(true);
                cmbTransMode.SelectedItem = ConnectData.GroupAxis.TransitionMode;
            }
        }

        private void EnableButtons(bool enable)
        {
            cmbTransMode.Enabled = enable;
            btnSelectPath.Enabled = enable;
            tbPath.Enabled = enable;
        }

       
        UInt32 MemHandle = 0xFFFFFFFF;
        private void btnSelectPath_Click(object sender, EventArgs e)
        {
            string path = tbPath.Text;
            try
            {
                if (MemHandle != 0xFFFFFFFF)
                {
                    ConnectData.GroupAxis.UnSelectPath(MemHandle, 1);
                }

                MemHandle = ConnectData.GroupAxis.SelectPath(MC_COORD_SYSTEM_ENUM.MC_ACS_COORD, path, 1);
                ConnectData.ToOutput("SelectPath command accepted  for group " + ConnectData.GroupAxis.AxisName);

                double[] lengths;
                uint num = ConnectData.GroupAxis.PathGetLengths(MemHandle, 0, 7, true, out lengths);
                if (num <= 0)
                {
                    ConnectData.ToOutput("Path lengths failed: " + num);
                    return;
                }
                ConnectData.ToOutput(string.Format("Path lengths ({0}) : ", num));
                for (int i = 0; i < num; i++)
                {
                    ConnectData.ToOutput(string.Format("    N {0} - {1}", i + 1, lengths[i]));
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
                return;
            }

            try
            {
                MemHandle = ConnectData.GroupAxis.MovePath(ConnectData.GroupAxis.TransitionParameter,
                                                           ConnectData.GroupAxis.CoordSystem,
                                                           (NC_TRANSITION_MODE_ENUM) cmbTransMode.SelectedItem,
                                                           ConnectData.GroupAxis.GroupBufferedMode,
                                                           MemHandle, ConnectData.GroupAxis.Superimposed, 1);
                ConnectData.ToOutput("MovePath command accepted  for group " + ConnectData.GroupAxis.AxisName);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
                string res, desc;
                MMCConnection.GetErrorCodeDescriptionByID( ConnectData.Handle, (short)ex.MMCError,0, out res, out desc);
                MessageBox.Show(res, desc);
                return;
            }

            ushort err =0 ;
            UInt32 state = 0;
            try
            {
                while( true )
                {
                    state = ConnectData.GroupAxis.GroupReadStatus(ref err);
                    if ((state & (UInt32)MC_STATE_GROUP.NC_GROUP_STANDBY) == (UInt32)MC_STATE_GROUP.NC_GROUP_STANDBY ||
                        (state & (UInt32)MC_STATE_GROUP.NC_GROUP_ERROR) == (UInt32)MC_STATE_GROUP.NC_GROUP_ERROR)
                        break;
                    System.Threading.Thread.Sleep(500);
                }
                
                ConnectData.GroupAxis.UnSelectPath( MemHandle, 1);
                ConnectData.ToOutput("UnSelectPath command accepted  for group " + ConnectData.GroupAxis.AxisName);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
            }
        }

        private void buttonInitTable_Click(object sender, EventArgs e)
        {
            try
            {
                if (MemHandle != 0xFFFFFFFF)
                {
                    MMCConnection.UnloadTable(ConnectData.Handle, MemHandle);
                }
                MemHandle = MMCConnection.InitTableEx(ConnectData.Handle, 1000, 2, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE,
                                          NC_MOTION_TABLE_TYPE_ENUM.eNC_TABLE_PVT_ARRAY, MC_COORD_SYSTEM_ENUM.MC_ACS_COORD,
                                          NC_ONLINE_SPLINE_MODE_ENUM.MC_QUINTIC_ON_PARAB_VT_DWELL, 60, 5,
                                          ConnectData.GroupAxis.AxisReference, 2, true, true, false, false, true);
//                 MemHandle = MMCConnection.InitTableEx(ConnectData.Handle, 1000, 2, MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE,
//                                          NC_MOTION_TABLE_TYPE_ENUM.eNC_TABLE_PVT_ARRAY, MC_COORD_SYSTEM_ENUM.MC_ACS_COORD,
//                                          NC_ONLINE_SPLINE_MODE_ENUM.MC_QUINTIC_ON_PARAB_VT_DWELL, 100, 10,
//                                          ConnectData.SingleAxis.AxisReference, 1, true, true, false, false, true);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
            }
        }

        private void btnWrite_Click(object sender, EventArgs e)
        {
            MMC_PARAMETER_LIST_ENUM Param = (MMC_PARAMETER_LIST_ENUM)cmbParam.SelectedItem;
            Double db = Double.Parse(tbValue.Text);
            try
            {
                ConnectData.GroupAxis.SetParameter(db, Param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
                return;
            }
        }

        private void btnWriteBool_Click(object sender, EventArgs e)
        {
            MMC_BOOLEAN_PARAMETER_LIST_ENUM Param = (MMC_BOOLEAN_PARAMETER_LIST_ENUM)cmbParam.SelectedItem;
            UInt32 db = UInt32.Parse(tbBoolValue.Text);
            try
            {
                ConnectData.GroupAxis.SetBoolParameter(db, Param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MmcException");
                return;
            }
        }

        private void btnReadBoolParam_Click(object sender, EventArgs e)
        {
            UInt32 res = 0;
            MMC_BOOLEAN_PARAMETER_LIST_ENUM Param = (MMC_BOOLEAN_PARAMETER_LIST_ENUM)cmbParam.SelectedItem;
            try
            {
                res = ConnectData.GroupAxis.GetBoolParameter(Param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            MessageBox.Show("Result of ReadBoolParameter = " + res.ToString() + "\nParam Name = " +
                            Param.ToString());
        }

        private void btnReadParam_Click(object sender, EventArgs e)
        {
            Double res = 0;
            MMC_PARAMETER_LIST_ENUM Param = (MMC_PARAMETER_LIST_ENUM)cmbParam.SelectedItem;
            try
            {
                res = ConnectData.GroupAxis.GetParameter( Param, 0);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            MessageBox.Show("Result of ReadParameter = " + res.ToString() + "\nParam Name = " +
                            Param.ToString());
        }

        private void btnWaitFor_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.GroupAxis == null)
                return;
            Double refernceValue = 1;
            Int32 paramID = 0, paramIndex = 0;
            MC_CONDITIONFB_OPERATION_TYPE condition = MC_CONDITIONFB_OPERATION_TYPE.MC_CONDITIONFB_OP_HIGHER;
            UInt16 sourceAxRef = 0;
            byte execute = 1;
            try
            {
                ConnectData.GroupAxis.WaitUntilConditionFB(refernceValue, paramID, paramIndex, condition, sourceAxRef,
                                                            execute);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }
    }
}
