﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class MovingControlVelocity : UserControl, IParentPanel
    {
        public MovingControlVelocity()
        {
            InitializeComponent();
            Visible = false;
        }
        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }

        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null || ConnectData.IsCANInternal == false)
                EnableButtons(false);
            else
                EnableButtons(true);
            
        }
        private void EnableButtons(bool enable)
        {
            btnPowerOn.Enabled = enable;
            btnPowerOff.Enabled = enable;
            btnStop.Enabled = enable;
            btnMoveVelocity.Enabled = enable;
            btnSetMode.Enabled = enable;
        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        private bool MotionPreview()
        {
            MC_STATE_SINGLE state = 0;
            uint plcStateMask = 0x7F8;//0x3F8;//0x7;//0x00000400
            uint validBitMask = 0x40000000;//0x3F8;//0x7;//0x00000400

            UInt32 rv = 0;

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }

            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }

            if ((state & MC_STATE_SINGLE.ERROR_STOP) == MC_STATE_SINGLE.ERROR_STOP)
            {
                try
                {
                    ConnectData.SingleAxis.Reset();
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MMC Exception");
                    EnableButtons(false);
                    MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                    ConnectData.Default();
                    return false;
                }
            }

            try
            {
                rv = ConnectData.SingleAxis.ReadStatus();
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                EnableButtons(false);
                MessageBox.Show("Please disconnect and reboot device", "Internal Communication Error");
                ConnectData.Default();
                return false;
            }
            if ((rv & validBitMask) == validBitMask)
            {
                state = (MC_STATE_SINGLE)(rv & plcStateMask);
            }
            else
            {
                MessageBox.Show("Illegal state recognized .Possible Reason - GMAS is in Preoperational mode.",
                                "Homing Control");
                //ConnectData.Default();
                return false;
            }

            
            return true;
        }

        private void OnVisibleChange(object sender, EventArgs e)
        {
            if( Visible == false )
            {
                if( ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null )
                {
                    try
                    {
                        ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
                    }
                    catch (MMCException ex)
                    {
                        MessageBox.Show(ex.ToString(), "MMC Exception");
                        return;
                    }
                }
            }
            else
            {
                if (ConnectData != null && ConnectData.IsValid && ConnectData.SingleAxis != null)
                {
                    //MotionPreview();
                    
                }
            }
        }

        private void btnPowerOn_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOn(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOn command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnPowerOff_Click(object sender, EventArgs e)
        {
            try
            {
                ConnectData.SingleAxis.PowerOff(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "PowerOff command successfully completed.";
            ConnectData.ToOutput(res);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;
            try
            {
                ConnectData.SingleAxis.Stop(MC_BUFFERED_MODE_ENUM.MC_BUFFERED_MODE);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Stop command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnMoveVelocity_Click(object sender, EventArgs e)
        {
            if (!MotionPreview())
                return;

            float vel;
            if (!float.TryParse(tbVelocity.Text, out vel))
            {
                MessageBox.Show("Invalid Velocity");
                return;
            }

            try
            {
                ConnectData.SingleAxis.MoveVelocity(vel, ConnectData.SingleAxis.HomeBufferedMode);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Move Velocity command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnSetMode_Click(object sender, EventArgs e)
        {
            OPM402 opMode = ConnectData.SingleAxis.GetOpMode();
            if (opMode != OPM402.OPM402_PROFILE_VELOCITY_MODE)
            {
                try
                {
                    ConnectData.SingleAxis.SetOpMode(OPM402.OPM402_PROFILE_VELOCITY_MODE);
                }
                catch (MMCException ex)
                {
                    MessageBox.Show(ex.ToString(), "MovingControlVelocity");
                }
                String res = "Operated Mode successfully changed to " + OPM402.OPM402_PROFILE_VELOCITY_MODE.ToString();
                ConnectData.ToOutput(res);
                return;
            }
            String rs = "Operated Mode already equal to " + OPM402.OPM402_PROFILE_VELOCITY_MODE.ToString();
            ConnectData.ToOutput(rs);
        }
    }
}
