﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;

namespace GMAS_Lib_test
{
    public partial class SingleAxisElmoManagmentControl : UserControl, IParentPanel
    {
        public SingleAxisElmoManagmentControl()
        {
            InitializeComponent();
        }
        private System.Windows.Forms.Panel panel;
        ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        public System.Windows.Forms.Panel GetPanel()
        {
            return panel;
        }
        public void Init(ConnectivityData data, System.Windows.Forms.Panel pnl)
        {
            ConnectData = data;
            panel = pnl;
            ConnectData.OnDataChange += new ConnectivityData.DataChange(ConnectData_OnDataChange);
            ConnectData_OnDataChange();
        }

        void ConnectData_OnDataChange()
        {
            if (ConnectData.IsValid == false || ConnectData.SingleAxis == null)
                EnableButtons(false);
            else
                EnableButtons(true);
         }

        private void EnableButtons(bool enable)
        {
            btnExecuteLabel.Enabled = enable;
            btnSetArr.Enabled = enable;
            btnSetParam.Enabled = enable;
            btnCallCommand.Enabled = enable;
            btnGetParam.Enabled = enable;
            btnGetArrayValue.Enabled = enable;
           
        }

        private void OnMouseOver(object sender, EventArgs e)
        {

        }

        private void OnLoad(object sender, EventArgs e)
        {
            Visible = true;
            Dock = DockStyle.Fill;
        }

        private void btnExecuteLabel_Click(object sender, EventArgs e)
        {
            byte[] lblArr = new byte[tbLabel.Text.Length];
            lblArr.Initialize();
            for (int i = 0; i < lblArr.Length; i++)
            {
                lblArr[i] = (byte)tbLabel.Text[i];
            }
            try
            {
                ConnectData.SingleAxis.ElmoExecute(lblArr, (byte) lblArr.Length);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo Execute command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnSetArr_Click(object sender, EventArgs e)
        {
            byte[] lblArrName = new byte[tbArrName.Text.Length];
            lblArrName.Initialize();
            for (int i = 0; i < lblArrName.Length; i++)
            {
                lblArrName[i] = (byte)tbArrName.Text[i];
            }
            short index;// = Int16.Parse(tbArrIndex.Text);
            if( !Int16.TryParse( tbArrIndex.Text, out index ) )
            {
                MessageBox.Show("Invalid Index");
                return;
            }
            Int32 val;// = 10;
            bool isInteger = true;
            if (!Int32.TryParse(tbValue.Text, out val))
            {
                isInteger = false;
                
            }
            float fVal;
            if (!float.TryParse(tbValue.Text, out fVal))
            {
                isInteger = false;
                MessageBox.Show("Invalid Value " + tbValue.Text);
                return;
            }

            try
            {
                if( isInteger)
                    ConnectData.SingleAxis.ElmoSetAsyncArray(lblArrName, index, ref val);
                else
                {
                    ConnectData.SingleAxis.ElmoSetAsyncArray(lblArrName, index, ref fVal);
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo SetArray command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        private void btnGetArrayValue_Click(object sender, EventArgs e)
        {
            if (tbArrName == null || tbArrName.Text.Length == 0)
                return;

            byte[] lblArrName = new byte[tbArrName.Text.Length];
            lblArrName.Initialize();
            for (int i = 0; i < lblArrName.Length; i++)
            {
                lblArrName[i] = (byte)tbArrName.Text[i];
            }
            short index;// = Int16.Parse(tbArrIndex.Text);
            if (!Int16.TryParse(tbArrIndex.Text, out index))
            {
                MessageBox.Show("Invalid Index");
                return;
            }

            try
            {
                if (ConnectData.IsValid)
                {
                    ConnectData.SingleAxis.ElmoGetAsyncIntArray(lblArrName, index);
                    ConnectData.AsyncRequst++;
                }
                //ConnectData.SingleAxis.ElmoGetAsyncFloatArray(lblArrName, index);
               
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        private void btnSetParam_Click(object sender, EventArgs e)
        {
            byte[] ParamName = new byte[tbParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbParamName.Text[i];
            }

            Int32 val;// = 10;
            if (!Int32.TryParse(tbParamValue.Text, out val))
            {
                MessageBox.Show("Invalid Value");
                return;
            }
            try
            {
                ConnectData.SingleAxis.ElmoSetAsyncParam(ParamName, ref val);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            String res = "Elmo SetParam command successfully Accepted.";
            ConnectData.ToOutput(res);
        }

        

        private void btnGetParam_Click(object sender, EventArgs e)
        {
            if (tbParamName == null || tbParamName.Text.Length == 0)
                return;
            byte[] ParamName = new byte[tbParamName.Text.Length];
            ParamName.Initialize();
            for (int i = 0; i < ParamName.Length; i++)
            {
                ParamName[i] = (byte)tbParamName.Text[i];
            }
            try
            {
                if (ConnectData.IsValid)
                {
                    ConnectData.SingleAxis.ElmoGetAsyncIntParam(ParamName);
                    ConnectData.AsyncRequst++;
                }
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            //String res = "Elmo SetParam command successfully Accepted.";
            //ConnectData.ToOutput(res);
        }

        

        private void btnCallCommand_Click(object sender, EventArgs e)
        {
            byte[] cmdArr = new byte[tbCmd.Text.Length];
            cmdArr.Initialize();
            for (int i = 0; i < cmdArr.Length; i++)
            {
                cmdArr[i] = (byte)tbCmd.Text[i];
            }
            try
            {
                ConnectData.SingleAxis.ElmoCallAsync( cmdArr );
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
            //String res = "Elmo Call command successfully Accepted.";
            //ConnectData.ToOutput(res);
        }

        private void btnWaitFor_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.SingleAxis == null)
                return;
            Double refernceValue = 456.321;
            Int32 paramID =(Int32) MMC_PARAMETER_LIST_ENUM.MMC_F_COMM_EV_USR_1_PARAM;
            Int32 paramIndex = 0;
            MC_CONDITIONFB_OPERATION_TYPE condition = MC_CONDITIONFB_OPERATION_TYPE.MC_CONDITIONFB_OP_EQUAL;
            UInt16 sourceAxRef = 1;
            byte execute = 1;
            try
            {
                ConnectData.SingleAxis.WaitUntilConditionFB(refernceValue, paramID, paramIndex, condition, sourceAxRef,
                                                            execute);
            }
            catch (MMCException ex)
            {
                MessageBox.Show(ex.ToString(), "MMC Exception");
                return;
            }
        }

        

        

        
    }
}
