﻿using System;
using System.Diagnostics;

namespace GMAS_Lib_test
{
    public class Stopper
    {
        #region Private Data

        private DateTime start;
        private DateTime finish;
        private bool isStarted;
        private bool isStoped;
        private TimeSpan interval;

        #endregion

        #region Construction

        public Stopper()
        {
            isStarted = false;
            isStoped = false;
        }

        public Stopper(bool start) : this()
        {
            if (start)
                Start();
        }

        #endregion

        #region Public Interface

        public void Start()
        {
            isStarted = true;
            isStoped = false;
            start = DateTime.Now;
        }

        public void Stop()
        {
            if (!isStarted)
                return;
            finish = DateTime.Now;
            isStoped = true;
            interval = finish.Subtract(start);
        }

        public int StopEx()
        {
            Stop();
            return Interval;
        }

        [Conditional("DEBUG")]
        public void DebugStart()
        {
            Start();
        }

        [Conditional("DEBUG")]
        public void DebugStop()
        {
            Stop();
        }

        /// <summary>
        /// Gets the interval.
        /// </summary>
        /// <value>The interval in milliseconds.</value>
        public int Interval
        {
            get
            {
                return isStoped
                           ? (((interval.Hours*60) + interval.Minutes)*60 + interval.Seconds)*1000 + interval.Milliseconds
                           : -1;
            }
        }

        /// <summary>
        /// Gets the interval.
        /// </summary>
        /// <value>The interval in milliseconds.</value>
        public int IntervalEx
        {
            get
            {
                int val = -1;
                if (isStoped)
                {
                    val = (int) (interval.Ticks/TimeSpan.TicksPerMillisecond);
                    if (interval.Ticks % TimeSpan.TicksPerMillisecond > TimeSpan.TicksPerMillisecond / 2)
                        val = val + 1;
                }
                return val;
            }
        }

        #endregion
    }
}

