﻿using System;
using System.Windows.Forms;
using ElmoMotionControl.GMAS.EASComponents.MMCLibDotNET;
using GMAS_Lib_test;

namespace MMCLibDotNET_Test_2010
{
    public partial class UserCommandControl : UserControl, IParentPanel
    {
        #region Construct

        public UserCommandControl()
        {
            InitializeComponent();
        }

        #endregion

        #region Parent Panel

        private Panel mainPanel;
        public Panel GetPanel()
        {
            return mainPanel;
        }

        private ConnectivityData connectData;
        public ConnectivityData ConnectData
        {
            get
            {
                return connectData;
            }
            set
            {
                connectData = value;
            }
        }
        
        public void Init(ConnectivityData data, Panel pnl)
        {
            ConnectData = data;
            mainPanel = pnl;
        }

        #endregion

        #region Command Control

        private string Program
        {
            get { return textBoxProgram.Text; }
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                short error = mmcUserCommand.StartProgram(Program);
                if (error != 0)
                {
                    MessageBox.Show(string.Format("Operation Start for program {0} failed with error {1}",
                                                  Program, error), "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Program {0} was started successfully", Program), "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                short error = mmcUserCommand.StopProgram(Program);
                if (error != 0)
                {
                    MessageBox.Show(string.Format("Operation Stop for program {0} failed with error {1}",
                                                  Program, error), "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Program {0} was stopped successfully", Program), "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        private void buttonIsExist_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                bool isExist;
                short error = mmcUserCommand.IsFileExist(Program, out isExist);
                if (error != 0)
                {
                    MessageBox.Show(string.Format("Operation IsExist for program {0} failed with error {1}",
                                                  Program, error), "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Program {0} {1}",
                        Program, isExist ? "exists" : "not exists"), "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        private void buttonIsRunning_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                bool isRunning;
                short error = mmcUserCommand.IsProgramRunning(Program, out isRunning);
                if (error != 0)
                {
                    MessageBox.Show(string.Format("Operation IsRunning for program {0} failed with error {1}",
                                                  Program, error), "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Program {0} {1}",
                        Program, isRunning ? "running" : "not running"), "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                short error = mmcUserCommand.RemoveFile(Program);
                if (error != 0)
                {
                    MessageBox.Show(string.Format("Operation Remove for program {0} failed with error {1}",
                                                  Program, error), "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Program {0} was removed successfully", Program), "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        private void buttonIsAnyGEXE_Click(object sender, EventArgs e)
        {
            if (ConnectData == null || ConnectData.Handle == 0)
            {
                MessageBox.Show("Invalid connection handler", "GMAS Programs");
                return;
            }
            MMCUserCommand mmcUserCommand = new MMCUserCommand(connectData.Handle);
            try
            {
                bool isRunning;
                string gexeProgram;
                short error = mmcUserCommand.IsAnyGexeRunning(out gexeProgram, out isRunning);
                if (error == 0)
                {
                    MessageBox.Show(isRunning
                                        ? string.Format("Program {0} is running", gexeProgram)
                                        : "No GEXE Program running",
                                    "GMAS Programs");
                }
                else
                {
                    MessageBox.Show(string.Format("Operation IsAnyGexeRunning failed with error {0}", error),
                                    "GMAS Programs");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GMAS Programs");
            }
        }

        #endregion
    }
}
